import os
import sys
import platform
import subprocess
import json
import shutil
from pathlib import Path

def check_os():
    """检查操作系统"""
    system = platform.system()
    if system != "Windows":
        print(f"⚠️ 警告: 当前操作系统为 {system}，本脚本主要针对 Windows 系统优化")
        if not input("是否继续安装? (y/n): ").lower().startswith('y'):
            sys.exit(1)
    return system

def check_python_version():
    """检查Python版本"""
    version = sys.version_info
    if version.major < 3 or (version.major == 3 and version.minor < 11):
        print(f"❌ 错误: Python版本必须大于等于3.11，当前版本为 {sys.version.split()[0]}")
        sys.exit(1)
    print(f"✅ Python版本检查通过: {sys.version.split()[0]}")

def check_and_install_uv():
    """检查和安装uv"""
    try:
        subprocess.run(["uv", "--version"], capture_output=True)
        print("✅ uv 已安装")
        # 检查uvx是否可用
        try:
            subprocess.run(["uvx", "--version"], capture_output=True)
            print("✅ uvx 已安装")
        except FileNotFoundError:
            print("⚙️ 正在配置 uvx...")
            subprocess.run([sys.executable, "-m", "uv", "pip", "install", "--system", "uv"], check=True)
            print("✅ uvx 配置成功")
    except FileNotFoundError:
        print("⚙️ 正在安装 uv...")
        try:
            subprocess.run([sys.executable, "-m", "pip", "install", "uv"], check=True)
            print("✅ uv 安装成功")
            # 安装完uv后配置uvx
            print("⚙️ 正在配置 uvx...")
            subprocess.run([sys.executable, "-m", "uv", "pip", "install", "--system", "uv"], check=True)
            print("✅ uvx 配置成功")
        except subprocess.CalledProcessError:
            print("❌ uv 安装失败")
            sys.exit(1)

def create_config():
    """创建默认配置文件"""
    config_dir = Path.home() / ".mcp2tcp"
    config_file = config_dir / "config.yaml"
    
    if not config_dir.exists():
        config_dir.mkdir(parents=True)
        print(f"✅ 创建配置目录: {config_dir}")

    if not config_file.exists():
        config_content = """serial:
  port: COM1  # 请修改为实际的COM端口号
  baud_rate: 115200

commands:
  set_pwm:
    command: "PWM {frequency}\\n"
    need_parse: false
    prompts:
      - "把PWM调到{value}"
      - "Set PWM to {value}%"
"""
        config_file.write_text(config_content, encoding='utf-8')
        print(f"✅ 创建配置文件: {config_file}")
        print("⚠️ 请修改配置文件中的COM端口号为实际值")
    else:
        print(f"ℹ️ 配置文件已存在: {config_file}")

def check_and_configure_claude():
    """检查和配置Claude桌面客户端"""
    claude_config_dir = Path.home() / "AppData/Roaming/Claude"
    if not claude_config_dir.exists():
        print(f"ℹ️ 未检测到Claude桌面客户端目录: {claude_config_dir}")
        return

    config_file = claude_config_dir / "claude_desktop_config.json"
    if not config_file.exists():
        print(f"ℹ️ Claude配置文件不存在: {config_file}")
        return

    try:
        with open(config_file, 'r', encoding='utf-8') as f:
            config = json.load(f)
    except json.JSONDecodeError:
        print("❌ Claude配置文件格式错误")
        return

    if "mcpServers" not in config:
        config["mcpServers"] = {}

    if "mcp2tcp" not in config["mcpServers"]:
        config["mcpServers"]["mcp2tcp"] = {
            "command": "uvx",
            "args": ["mcp2tcp"]
        }
        with open(config_file, 'w', encoding='utf-8') as f:
            json.dump(config, f, indent=2)
        print("✅ 已添加mcp2tcp配置到Claude")
    else:
        print("ℹ️ Claude已配置mcp2tcp")

def check_vscode():
    """检查VSCode安装"""
    vscode_path = Path.home() / "AppData/Local/Programs/Microsoft VS Code"
    if vscode_path.exists():
        print("""
ℹ️ 检测到VSCode安装
请在VSCode中添加以下MCP服务器配置：
{
    "mcp2tcp": {
        "command": "uvx",
        "args": ["mcp2tcp"]
    }
}
""")
    else:
        print("ℹ️ 未检测到VSCode安装")

def main():
    print("=== mcp2tcp 安装程序 ===")
    
    # 1. 检查操作系统
    system = check_os()
    print(f"✅ 操作系统: {system}")
    
    # 2. 检查Python版本
    check_python_version()
    
    # 3. 检查和安装uv/uvx
    check_and_install_uv()
    
    # 4. 创建配置文件
    create_config()
    
    # 5. 检查和配置Claude
    check_and_configure_claude()
    
    # 6. 检查VSCode
    check_vscode()
    
    print("\n✨ 安装完成！")
    print("📝 请确保：")
    print("1. 修改配置文件中的COM端口号")
    print("2. 检查Claude或VSCode的MCP服务器配置")
    print("3. 重启Claude或VSCode以使配置生效")
    print("\n💡 提示：mcp2tcp 将在首次运行时自动下载")

if __name__ == "__main__":
    main()



import pytest
from mcp2tcp.server import create_app

@pytest.fixture
def app():
    app = create_app()
    return app

def test_set_pwm_endpoint(app):
    with app.test_client() as client:
        # Test valid request
        response = client.post('/set-pwm', json={'frequency': 50})
        assert response.status_code == 200
        assert response.json['status'] == 'success'

        # Test invalid frequency
        response = client.post('/set-pwm', json={'frequency': 101})
        assert response.status_code == 400
        assert 'error' in response.json

        # Test missing frequency parameter
        response = client.post('/set-pwm', json={})
        assert response.status_code == 400
        assert 'error' in response.json



import os
import sys
import platform
import subprocess
import json
from pathlib import Path

def check_os():
    """检查操作系统"""
    system = platform.system()
    if system != "Darwin":
        print(f"❌ 错误: 此脚本仅支持 MacOS 系统，当前系统为 {system}")
        sys.exit(1)
    print(f"✅ 操作系统: MacOS {platform.mac_ver()[0]}")
    return system

def check_python_version():
    """检查Python版本"""
    version = sys.version_info
    if version.major < 3 or (version.major == 3 and version.minor < 11):
        print(f"❌ 错误: Python版本必须大于等于3.11，当前版本为 {sys.version.split()[0]}")
        sys.exit(1)
    print(f"✅ Python版本检查通过: {sys.version.split()[0]}")

def check_homebrew():
    """检查是否安装了Homebrew"""
    try:
        subprocess.run(["brew", "--version"], capture_output=True)
        print("✅ Homebrew 已安装")
    except FileNotFoundError:
        print("❌ 请先安装 Homebrew")
        print("💡 安装命令: /bin/bash -c \"$(curl -fsSL https://raw.githubusercontent.com/Homebrew/install/HEAD/install.sh)\"")
        sys.exit(1)

def check_and_install_uv():
    """检查和安装uv"""
    try:
        subprocess.run(["uv", "--version"], capture_output=True)
        print("✅ uv 已安装")
        # 检查uvx是否可用
        try:
            subprocess.run(["uvx", "--version"], capture_output=True)
            print("✅ uvx 已安装")
        except FileNotFoundError:
            print("⚙️ 正在配置 uvx...")
            subprocess.run([sys.executable, "-m", "uv", "pip", "install", "--system", "uv"], check=True)
            print("✅ uvx 配置成功")
    except FileNotFoundError:
        print("⚙️ 正在安装 uv...")
        try:
            # 使用 Homebrew 安装 uv
            subprocess.run(["brew", "install", "astral-sh/tap/uv"], check=True)
            print("✅ uv 安装成功")
            # 安装完uv后配置uvx
            print("⚙️ 正在配置 uvx...")
            subprocess.run([sys.executable, "-m", "uv", "pip", "install", "--system", "uv"], check=True)
            print("✅ uvx 配置成功")
        except subprocess.CalledProcessError:
            print("❌ uv 安装失败")
            sys.exit(1)

def create_config():
    """创建默认配置文件"""
    config_dir = Path.home() / ".mcp2tcp"
    config_file = config_dir / "config.yaml"
    
    if not config_dir.exists():
        config_dir.mkdir(parents=True)
        print(f"✅ 创建配置目录: {config_dir}")

    if not config_file.exists():
        config_content = """serial:
  port: /dev/tty.usbserial-*  # TCP设备名，支持通配符
  baud_rate: 115200

commands:
  set_pwm:
    command: "PWM {frequency}\\n"
    need_parse: false
    prompts:
      - "把PWM调到{value}"
      - "Set PWM to {value}%"
"""
        config_file.write_text(config_content, encoding='utf-8')
        print(f"✅ 创建配置文件: {config_file}")
        print("⚠️ 请修改配置文件中的TCP设备名为实际值")
    else:
        print(f"ℹ️ 配置文件已存在: {config_file}")

def check_and_configure_claude():
    """检查和配置Claude桌面客户端"""
    claude_config_dir = Path.home() / "Library/Application Support/Claude"
    if not claude_config_dir.exists():
        print(f"ℹ️ 未检测到Claude桌面客户端目录: {claude_config_dir}")
        return

    config_file = claude_config_dir / "claude_desktop_config.json"
    if not config_file.exists():
        print(f"ℹ️ Claude配置文件不存在: {config_file}")
        return

    try:
        with open(config_file, 'r', encoding='utf-8') as f:
            config = json.load(f)
    except json.JSONDecodeError:
        print("❌ Claude配置文件格式错误")
        return

    if "mcpServers" not in config:
        config["mcpServers"] = {}

    if "mcp2tcp" not in config["mcpServers"]:
        config["mcpServers"]["mcp2tcp"] = {
            "command": "uvx",
            "args": ["mcp2tcp"]
        }
        with open(config_file, 'w', encoding='utf-8') as f:
            json.dump(config, f, indent=2)
        print("✅ 已添加mcp2tcp配置到Claude")
    else:
        print("ℹ️ Claude已配置mcp2tcp")

def check_vscode():
    """检查VSCode安装"""
    vscode_path = Path("/Applications/Visual Studio Code.app")
    if vscode_path.exists():
        print("""
ℹ️ 检测到VSCode安装
请在VSCode中添加以下MCP服务器配置：
{
    "mcp2tcp": {
        "command": "uvx",
        "args": ["mcp2tcp"]
    }
}
""")
    else:
        print("ℹ️ 未检测到VSCode安装")

def check_serial_devices():
    """检查TCP设备"""
    devices = list(Path("/dev").glob("tty.usbserial-*"))
    if devices:
        print("\n检测到以下TCP设备：")
        for device in devices:
            print(f"- {device}")
        print("💡 请在配置文件中使用正确的设备名")
    else:
        print("\n⚠️ 未检测到TCP设备，请确保：")
        print("1. 设备已正确连接")
        print("2. 已安装TCP驱动")
        print("💡 常用TCP芯片驱动：")
        print("- CH340/CH341: https://www.wch.cn/downloads/CH341SER_MAC_ZIP.html")
        print("- CP210x: https://www.silabs.com/developers/usb-to-uart-bridge-vcp-drivers")
        print("- FTDI: https://ftdichip.com/drivers/vcp-drivers/")

def main():
    print("=== mcp2tcp MacOS 安装程序 ===")
    
    # 1. 检查操作系统
    system = check_os()
    
    # 2. 检查Python版本
    check_python_version()
    
    # 3. 检查Homebrew
    check_homebrew()
    
    # 4. 检查和安装uv/uvx
    check_and_install_uv()
    
    # 5. 创建配置文件
    create_config()
    
    # 6. 检查和配置Claude
    check_and_configure_claude()
    
    # 7. 检查VSCode
    check_vscode()
    
    # 8. 检查TCP设备
    check_serial_devices()
    
    print("\n✨ 安装完成！")
    print("📝 请确保：")
    print("1. 修改配置文件中的TCP设备名")
    print("2. 检查Claude或VSCode的MCP服务器配置")
    print("3. 重启Claude或VSCode以使配置生效")
    print("\n💡 提示：")
    print("- mcp2tcp 将在首次运行时自动下载")
    print("- TCP设备名通常为 /dev/tty.usbserial-* 格式")
    print("- 如遇到权限问题，请确保当前用户有TCP设备的读写权限")

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# ====================================================
# Project: mcp2tcp
# Description: Installation script for Ubuntu/Raspberry Pi
# Repository: https://github.com/mcp2everything/mcp2tcp.git
# License: MIT License
# Author: mcp2everything
# Copyright (c) 2024 mcp2everything
# ====================================================

import os
import sys
import json
import subprocess
from pathlib import Path
import shutil

def get_uv_path():
    """Get uv executable path."""
    # 检查是否已安装uv
    try:
        result = subprocess.run(['which', 'uv'], capture_output=True, text=True)
        if result.returncode == 0:
            return result.stdout.strip()
    except:
        pass
    return None

def install_uv():
    """Install uv package manager."""
    print("Installing uv package manager...")
    try:
        # 使用curl安装uv
        curl_command = 'curl -LsSf https://astral.sh/uv/install.sh | sh'
        subprocess.run(curl_command, shell=True, check=True)
        
        # 添加uv到PATH
        home = str(Path.home())
        bashrc_path = os.path.join(home, '.bashrc')
        with open(bashrc_path, 'a') as f:
            f.write('\n# uv package manager\nexport PATH="$HOME/.cargo/bin:$PATH"\n')
        
        print("uv installed successfully!")
        return True
    except Exception as e:
        print(f"Error installing uv: {e}")
        return False

def install_mcp2tcp():
    """Install mcp2tcp package."""
    try:
        # 创建虚拟环境并安装包
        subprocess.run(['uv', 'venv', '.venv'], check=True)
        subprocess.run(['.venv/bin/uv', 'pip', 'install', 'mcp2tcp'], check=True)
        return True
    except Exception as e:
        print(f"Error installing mcp2tcp: {e}")
        return False

def configure_claude_desktop():
    """Configure Claude Desktop with mcp2tcp."""
    try:
        home = str(Path.home())
        config_dir = os.path.join(home, '.config', 'claude-desktop')
        os.makedirs(config_dir, exist_ok=True)
        
        config_file = os.path.join(config_dir, 'config.json')
        config = {
            "mcpServers": {
                "mcp2tcp": {
                    "command": "uvx",
                    "args": ["mcp2tcp"]
                }
            }
        }
        
        # 如果配置文件已存在，则更新而不是覆盖
        if os.path.exists(config_file):
            with open(config_file, 'r') as f:
                existing_config = json.load(f)
            existing_config.setdefault('mcpServers', {})
            existing_config['mcpServers']['mcp2tcp'] = config['mcpServers']['mcp2tcp']
            config = existing_config
        
        with open(config_file, 'w') as f:
            json.dump(config, f, indent=4)
            
        print("Claude Desktop configured successfully!")
        return True
    except Exception as e:
        print(f"Error configuring Claude Desktop: {e}")
        return False

def setup_config():
    """Setup configuration files."""
    try:
        home = str(Path.home())
        config_dir = os.path.join(home, '.mcp2tcp')
        os.makedirs(config_dir, exist_ok=True)
        
        # 复制默认配置文件
        default_config = os.path.join(os.path.dirname(__file__), 'config.yaml')
        user_config = os.path.join(config_dir, 'config.yaml')
        
        if os.path.exists(default_config):
            shutil.copy2(default_config, user_config)
            print(f"Configuration file copied to: {user_config}")
        return True
    except Exception as e:
        print(f"Error setting up configuration: {e}")
        return False

def main():
    """Main installation process."""
    print("Starting mcp2tcp installation for Ubuntu/Raspberry Pi...")
    
    # 检查Python版本
    if sys.version_info < (3, 11):
        print("Error: Python 3.11 or higher is required")
        sys.exit(1)
    
    # 安装uv（如果需要）
    if not get_uv_path():
        if not install_uv():
            print("Failed to install uv package manager")
            sys.exit(1)
    
    # 安装mcp2tcp
    if not install_mcp2tcp():
        print("Failed to install mcp2tcp")
        sys.exit(1)
    
    # 配置Claude Desktop
    if not configure_claude_desktop():
        print("Warning: Failed to configure Claude Desktop")
    
    # 设置配置文件
    if not setup_config():
        print("Warning: Failed to setup configuration files")
    
    print("\nInstallation completed!")
    print("\nTo use mcp2tcp:")
    print("1. Activate the virtual environment:")
    print("   source .venv/bin/activate")
    print("2. Run the server:")
    print("   uv run mcp2tcp")
    print("\nFor more information, visit: https://github.com/mcp2everything/mcp2tcp")

if __name__ == "__main__":
    main()



# ====================================================
# Project: mcp2tcp
# Description: A protocol conversion tool that enables 
#              hardware devices to communicate with 
#              large language models (LLM) through serial ports.
# Repository: https://github.com/mcp2everything/mcp2tcp.git
# License: MIT License
# Author: mcp2everything
# Copyright (c) 2024 mcp2everything
#
# Permission is hereby granted, free of charge, to any person 
# obtaining a copy of this software and associated documentation 
# files (the "Software"), to deal in the Software without restriction, 
# including without limitation the rights to use, copy, modify, merge, 
# publish, distribute, sublicense, and/or sell copies of the Software, 
# and to permit persons to whom the Software is furnished to do so, 
# subject to the following conditions:
#
# The above copyright notice and this permission notice shall be 
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
# DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
# IN THE SOFTWARE.
# ====================================================
from . import server
import asyncio
import argparse


def main():
    """Main entry point for the package."""
    parser = argparse.ArgumentParser(description='mcp2tcp Server')
    parser.add_argument('--config', 
                       default="default",
                       help='Configuration name (without _config.yaml suffix)')
    
    args = parser.parse_args()
    asyncio.run(server.main(args.config))


# Expose important items at package level
__version__ = "0.1.0"
__all__ = ['main', 'server']



import socket
import json
import time

'''
tcp_server.py
模拟硬件设备的TCP服务器，根据接收到的命令返回相应的响应。
支持的命令：
- CMD_PWM {frequency}: 设置PWM频率
- CMD_PICO_INFO: 查询Pico板信息
- CMD_LED {state}: 控制LED状态
'''

class DeviceState:
    def __init__(self):
        self.pwm_frequency = 0
        self.led_state = "off"
        self.start_time = time.time()
    
    def set_pwm(self, frequency):
        try:
            self.pwm_frequency = int(frequency)
            return f"CMD PWM frequency set to {frequency}Hz\r\n"
        except ValueError:
            return f"CMD Error: Invalid frequency value\r\n"
    
    def get_pico_info(self):
        uptime = int(time.time() - self.start_time)
        info = {
            "device": "Raspberry Pi Pico",
            "firmware_version": "1.0.0",
            "current_pwm": self.pwm_frequency,
            "led_state": self.led_state,
            "uptime_seconds": uptime
        }
        return f"CMD {json.dumps(info)}\r\n"
    
    def set_led(self, state):
        if state.lower() in ["on", "off"]:
            self.led_state = state.lower()
            return f"CMD LED state set to {state}\r\n"
        return f"CMD Error: Invalid LED state. Use 'on' or 'off'\r\n"

def handle_command(device_state: DeviceState, command: str) -> str:
    """处理接收到的命令并返回响应"""
    command = command.strip()
    
    # 处理PWM命令
    if command.startswith("CMD_PWM"):
        try:
            frequency = command.split()[1]
            return device_state.set_pwm(frequency)
        except IndexError:
            return "CMD Error: Missing frequency parameter\r\n"
    
    # 处理Pico信息查询命令
    elif command == "CMD_PICO_INFO":
        return device_state.get_pico_info()
    
    # 处理LED控制命令
    elif command.startswith("CMD_LED"):
        try:
            state = command.split()[1]
            return device_state.set_led(state)
        except IndexError:
            return "CMD Error: Missing LED state parameter\r\n"
    
    # 未知命令
    else:
        return f"CMD Error: Unknown command '{command}'\r\n"

def start_server(host='127.0.0.1', port=9999):
    device_state = DeviceState()
    
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as server_socket:
        server_socket.bind((host, port))
        server_socket.listen(1)
        print(f"TCP Server listening on {host}:{port}")
        print("Supported commands:")
        print("- CMD_PWM {frequency}")
        print("- CMD_PICO_INFO")
        print("- CMD_LED {state}")

        while True:
            client_socket, client_address = server_socket.accept()
            with client_socket:
                print(f"Client connected from {client_address}")
                while True:
                    try:
                        data = client_socket.recv(1024)
                        if not data:
                            break
                        
                        command = data.decode('ascii').strip()
                        print(f"Received: {command}")
                        
                        # 处理命令并发送响应
                        response = handle_command(device_state, command)
                        client_socket.sendall(response.encode('ascii'))
                        print(f"Sent: {response.strip()}")
                        
                    except UnicodeDecodeError:
                        error_response = "CMD Error: Invalid ASCII data\r\n"
                        client_socket.sendall(error_response.encode('ascii'))
                    except Exception as e:
                        error_response = f"CMD Error: {str(e)}\r\n"
                        client_socket.sendall(error_response.encode('ascii'))
                
                print(f"Client {client_address} disconnected")

if __name__ == "__main__":
    start_server()



# ====================================================
# Project: mcp2tcp
# Description: A protocol conversion tool that enables 
#              hardware devices to communicate with 
#              large language models (LLM) through serial ports.
# Repository: https://github.com/mcp2everything/mcp2tcp.git
# License: MIT License
# Author: mcp2everything
# Copyright (c) 2024 mcp2everything
#
# Permission is hereby granted, free of charge, to any person 
# obtaining a copy of this software and associated documentation 
# files (the "Software"), to deal in the Software without restriction, 
# including without limitation the rights to use, copy, modify, merge, 
# publish, distribute, sublicense, and/or sell copies of the Software, 
# and to permit persons to whom the Software is furnished to do so, 
# subject to the following conditions:
#
# The above copyright notice and this permission shall be 
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
# DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
# IN THE SOFTWARE.
# ====================================================
from typing import Any, Optional, Tuple, Dict, List
import asyncio
from mcp.server.models import InitializationOptions
import mcp.types as types
from mcp.server import NotificationOptions, Server
import mcp.server.stdio
import logging
import yaml
import os
from dataclasses import dataclass, field
import time
import socket

# 设置日志级别为 DEBUG
logging.basicConfig(
    level=logging.DEBUG,  # 改为 DEBUG 级别以显示更多信息
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s'
)
logger = logging.getLogger(__name__)

# 添加版本号常量
VERSION = "0.1.0"  # 添加了自动\r\n和更详细的错误信息

# 创建MCP服务器实例
server = Server("mcp2tcp")
config = None

@server.list_tools()
async def handle_list_tools() -> List[types.Tool]:
    """List available tools."""
    if config is None:
        return []
        
    tools = []
    for cmd_id, command in config.commands.items():
        # 从命令字符串中提取命令名（去掉CMD_前缀）
        cmd_name = command.command.split()[0].replace("CMD_", "").lower()
        
        # 构建参数描述
        properties = {}
        required = []
        
        # 使用配置文件中的参数定义
        if hasattr(command, 'parameters'):
            for param in command.parameters:
                properties[param['name']] = {
                    "type": param['type'],
                    "description": param['description'],
                    **({"enum": param['enum']} if 'enum' in param else {})
                }
                if param.get('required', False):
                    required.append(param['name'])
        else:
            # 如果没有参数定义，从命令字符串中提取
            import re
            param_names = re.findall(r'\{(\w+)\}', command.command)
            for param_name in param_names:
                properties[param_name] = {
                    "type": "string",
                    "description": f"Parameter {param_name} for the {cmd_name} command",
                    "examples": [p.format(**{param_name: "value"}) for p in command.prompts if "{" + param_name + "}" in p]
                }
                required.append(param_name)

        tool = types.Tool(
            name=cmd_name,  # 使用命令名作为工具名
            description=command.prompts[0] if command.prompts else f"Execute {cmd_name} command",
            inputSchema={
                "type": "object",
                "properties": properties,
                "required": required,
                "additionalProperties": False
            }
        )
        tools.append(tool)
        logger.debug(f"Registered tool: {cmd_name} with parameters: {properties}")
    
    return tools

@server.call_tool()
async def handle_call_tool(name: str, arguments: Dict[str, Any] | None) -> List[types.TextContent]:
    """Handle tool execution requests."""
    if config is None:
        return [types.TextContent(
            type="text",
            text="Error: Configuration not loaded"
        )]
        
    try:
        logger.info(f"Tool call received - Name: {name}, Arguments: {arguments}")
        
        # 查找对应的命令
        cmd_found = None
        for cmd_id, command in config.commands.items():
            cmd_name = command.command.split()[0].replace("CMD_", "").lower()
            if cmd_name == name:
                cmd_found = command
                break
        
        if cmd_found is None:
            error_msg = f"Error: Unknown tool '{name}'\n"
            error_msg += "Please check:\n"
            error_msg += "1. Tool name is correct\n"
            error_msg += "2. Tool is configured in config.yaml"
            logger.error(error_msg)
            return [types.TextContent(
                type="text",
                text=error_msg
            )]

        if arguments is None:
            arguments = {}
        
        # 验证必需的参数
        import re
        param_names = re.findall(r'\{(\w+)\}', cmd_found.command)
        missing_params = [param for param in param_names if param not in arguments]
        if missing_params:
            error_msg = f"Error: Missing required parameters: {', '.join(missing_params)}\n"
            error_msg += "Please provide all required parameters."
            logger.error(error_msg)
            return [types.TextContent(
                type="text",
                text=error_msg
            )]
        
        # 发送命令并等待响应
        try:
            response = tcp_connection.send_command(cmd_found, arguments)
            logger.debug(f"Command response: {response}")
            return response
        except ConnectionError as e:
            error_msg = f"Error: Connection failed - {str(e)}\n"
            error_msg += "Please check:\n"
            error_msg += "1. TCP server is running\n"
            error_msg += "2. Connection settings are correct"
            logger.error(error_msg)
            return [types.TextContent(
                type="text",
                text=error_msg
            )]
        except TimeoutError as e:
            error_msg = f"Error: Command timeout - {str(e)}\n"
            error_msg += "Please check:\n"
            error_msg += "1. Device is responding\n"
            error_msg += "2. Timeout settings are appropriate"
            logger.error(error_msg)
            return [types.TextContent(
                type="text",
                text=error_msg
            )]
        
    except Exception as e:
        error_msg = f"Error: {str(e)}\n"
        error_msg += "Please check:\n"
        error_msg += "1. Configuration is correct\n"
        error_msg += "2. Device is functioning properly"
        logger.error(f"Unexpected error: {str(e)}", exc_info=True)
        return [types.TextContent(
            type="text",
            text=error_msg
        )]

@dataclass
class Command:
    """Configuration for a serial command."""
    command: str
    need_parse: bool
    data_type: str
    prompts: List[str]

@dataclass
class Config:
    """Configuration for mcp2tcp service."""
    remote_ip: Optional[str] = None
    port: int = 12345
    connect_timeout: float = 3.0
    receive_timeout: float = 3.0
    timeout: float = 1.0
    read_timeout: float = 1.0
    response_start_string: str = "OK"  # 新增：可配置的应答开始字符串
    communication_type: str = "client"  # 新增：通信类型
    commands: Dict[str, Command] = field(default_factory=dict)

    @staticmethod
    def load(config_path: str = "config.yaml") -> 'Config':
        """Load configuration from YAML file."""
        try:
            logger.info(f"Opening configuration file: {config_path}")
            with open(config_path, 'r') as f:
                data = yaml.safe_load(f)
                logger.info("Successfully parsed YAML configuration")
            
            # 加载 TCP 配置
            tcp_config = data.get('tcp', {})
            logger.info("Loading TCP configuration...")
            config = Config(
                remote_ip=tcp_config.get('remote_ip'),
                port=tcp_config.get('port', 12345),
                connect_timeout=tcp_config.get('connect_timeout', 3.0),
                receive_timeout=tcp_config.get('receive_timeout', 3.0),
                response_start_string=tcp_config.get('response_start_string', 'OK'),
                communication_type=tcp_config.get('communication_type', 'client')
            )
            logger.info("TCP configuration loaded")
            
            # 加载命令配置
            logger.info("Loading commands configuration...")
            commands_count = 0
            for cmd_id, cmd_data in data.get('commands', {}).items():
                logger.info(f"Loading command: {cmd_id}")
                raw_command = cmd_data.get('command', '')
                logger.debug(f"Command string: '{raw_command}'")
                config.commands[cmd_id] = Command(
                    command=raw_command,
                    need_parse=cmd_data.get('need_parse', False),
                    data_type=cmd_data.get('data_type', 'ascii'),
                    prompts=cmd_data.get('prompts', [])
                )
                commands_count += 1
            logger.info(f"Loaded {commands_count} commands")
            
            return config
            
        except Exception as e:
            logger.error(f"Error loading config: {e}")
            raise

class TCPConnection:
    """TCP connection manager."""
    
    def __init__(self):
        self.socket: Optional[socket.socket] = None
        self.remote_ip: str = None
        self.port: int = None
        self.connect_timeout: float = None
        self.receive_timeout: float = None
        self.response_start_string: str = None

    def connect(self) -> bool:
        """Attempt to connect to the TCP server."""
        try:
            if self.socket:
                self.socket.close()
            self.socket = socket.create_connection(
                (self.remote_ip, self.port), 
                timeout=self.connect_timeout
            )
            logger.info(f"Connected to TCP server at {self.remote_ip}:{self.port}")
            return True
        except Exception as e:
            logger.error(f"Failed to connect to TCP server: {str(e)}")
            return False

    def send_command(self, command: Command, arguments: dict[str, Any] | None) -> list[types.TextContent]:
        """Send a command to the TCP server and return result according to MCP protocol."""
        try:
            if not self.socket:
                if not self.connect():
                    return [types.TextContent(
                        type="text",
                        text=f"Failed to connect to TCP server at {self.remote_ip}:{self.port}"
                    )]
            if command.data_type == "ascii":
                # 准备命令
                cmd_str = command.command.format(**arguments)
                # 确保命令以\r\n结尾
                cmd_str = cmd_str.rstrip() + '\r\n'  # 移除可能的空白字符，强制添加\r\n
                command_bytes = cmd_str.encode()
                logger.info(f"Sending command: {cmd_str.strip()}")
                logger.info(f"Sent command: {command_bytes.strip().decode('ascii')}")
                self.socket.sendall(command_bytes)

            elif command.data_type == "hex":
                command_bytes = bytes.fromhex(command.command.replace(" ", ""))
                logger.info(f"Sent command: {command.command}")
                self.socket.sendall(command_bytes)
            self.socket.settimeout(self.receive_timeout)
            responses = []
            while True:
                try:
                    response = self.socket.recv(4096)
                    if response:
                        logger.debug(f"Received data: {response}")
                        responses.append(response)
                        if command.data_type == "ascii" and response.endswith(b'\r\n'):
                            break
                        elif command.data_type == "hex":
                            break
                    else:
                        break
                except socket.timeout as e:
                    logger.error(f"TCP receive timeout: {str(e)}")
                    return [types.TextContent(
                        type="text",
                        text=f"TCP receive timeout: {str(e)}"
                    )]

            if not responses:
                return [types.TextContent(
                    type="text",
                    text=f"No response received from TCP server within {self.receive_timeout} seconds"
                )]

            first_response = responses[0].decode().strip()
            logger.info(f"Received response: {first_response}")

            if self.response_start_string in first_response:
                return [types.TextContent(
                    type="text",
                    text=first_response
                )]
            else:
                return [types.TextContent(
                    type="text",
                    text=f"Invalid response: {first_response}"
                )]

        except socket.timeout as e:
            logger.error(f"TCP send timeout: {str(e)}")
            return [types.TextContent(
                type="text",
                text=f"TCP send timeout: {str(e)}"
            )]
        except Exception as e:
            logger.error(f"TCP error: {str(e)}")
            return [types.TextContent(
                type="text",
                text=f"TCP error: {str(e)}"
            )]

    def close(self) -> None:
        """Close the TCP connection if open."""
        if self.socket:
            self.socket.close()
            logger.info(f"Closed TCP connection to {self.remote_ip}:{self.port}")
            self.socket = None

tcp_connection = TCPConnection()

def send_command(command, arguments):
    response = tcp_connection.send_command(command,arguments)
    logger.info(f"Received response: {response}")
    print(f"Received response: {response}")
    return response

async def main(config_name: str = None) -> None:
    """Run the MCP server.
    
    Args:
        config_name: Optional configuration name. If not provided, uses default config.yaml
    """
    try:
        # 加载配置
        config_path = config_name if config_name else "config.yaml"
        if not os.path.isfile(config_path):
            config_path = os.path.join(os.path.dirname(__file__), '..', '..', 'config.yaml')
        
        logger.info(f"Loading configuration from {config_path}")
        if not os.path.isfile(config_path):
            logger.error(f"Configuration file not found: {config_path}")
            raise FileNotFoundError(f"Configuration file not found: {config_path}")
            
        global config
        config = Config.load(config_path)
        logger.info("Configuration loaded successfully")
        logger.info(f"TCP Remote IP: {config.remote_ip}")
        logger.info(f"TCP Port: {config.port}")
        logger.info(f"Available commands: {list(config.commands.keys())}")
        
        tcp_connection.remote_ip = config.remote_ip
        tcp_connection.port = config.port
        tcp_connection.connect_timeout = config.connect_timeout
        tcp_connection.receive_timeout = config.receive_timeout * 5  # 增加接收超时时间
        tcp_connection.response_start_string = config.response_start_string
        
        # 运行 MCP 服务器
        logger.info("Starting MCP server...")
        async with mcp.server.stdio.stdio_server() as (read_stream, write_stream):
            await server.run(
                read_stream,
                write_stream,
                InitializationOptions(
                    server_name="mcp2tcp",
                    server_version="0.1.0",
                    capabilities=server.get_capabilities(
                        notification_options=NotificationOptions(),
                        experimental_capabilities={},
                    ),
                ),
            )
            
    except KeyboardInterrupt:
        logger.info("Received keyboard interrupt")
    except Exception as e:
        logger.error(f"Error in main: {e}")
        raise

if __name__ == "__main__":
    import sys
    config_name = sys.argv[1] if len(sys.argv) > 1 else None
    asyncio.run(main(config_name))